// ========================================================================
//
// mipsdefs.h
//
// Important constants and register content masks/ values for MIPS r3000
// based processors
//
// NOTE: This file may included by assembly language too, so
//       it can't include other files or use any C/C++ types!
//
//
// Copyright (c) 1999 Frank Somers, Ground Zero Development
// <frank@g0dev.com>
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation; either version 2 of the License, or (at your
// option) any later version.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// ========================================================================

#ifndef  MIPSDEFS_H
#define  MIPSDEFS_H


// ------------------------------------------------------------------------
//
// ISA features
//
//
// MIPS_ISA
//    Define this to be a value 1..n which selects the highest MIPS ISA
//    for which the processor is completely User ISA compatible
//
// MIPS_HAS_BRANCH_LIKELY
//    Implicifly supported if ISA is >= 2.  Use this for hybrids that are
//    MipsI plus 'branch likely' insns from MipsII
//
// MIPS_HAS_FPU
//    Define this if there is an FPU present in COP1
//    XXX not implemented yet
//
// XXX Clean up: MIPS_R3900 refs in insgen.cpp


#if   defined (EMUCFG_CPU_3910)
   // MipsI core plus 'branch likely' plus MADD

   #define  MIPS_ISA                      1
   #define  MIPS_HAS_BRANCH_LIKELY
#else
   #error "Need ISA config params"
#endif


// Derive some useful config info from ISA config

#if   MIPS_ISA <= 2
   #define  MIPS_32BIT
#else
   // XXX This isn't supported yet!
   // This currently just serves to highlight areas that will need
   // work if 64-bit support is to be added
   #define  MIPS_64BIT
#endif


// How many coprocessors are supported by the ISA?

#if   MIPS_ISA <= 2
   #define  MIPS_COP_COUNT                4
#else
   // MIPSII and above have reused some Cop3 insn opcodes

   #define  MIPS_COP_COUNT                3
#endif



// Just to avoid magic numbers, define instruction opcode size

#define  MIPS_SIZEOF_OPCODE               4


// ------------------------------------------------------------------------
//
// Power/ speed management
//
// Several embedded MIPS variants have mechanisms to adjust the core
// clock speed and enter/ leave a halt state
//
// MIPS_CLOCK_SPEED
//    This should specify the maximum INSTRUCTION clock rate the processor
//    is capable of in the emulated device
//
// MIPS_SELECTABLE_CLOCK_RATES
//    If the processor supports selectable clock rates (under software
//    control), then this should be defined as the number of rates that
//    are available.  EG, if 4 clock divider rations are available, this
//    is defined as 4
//
// MIPS_HALTABLE_CPU
//    Define this if the processor can be halted under software control.
//    This normally means it can be woken up by an external interrupt.
//    I don't know of other possiblilities at present.
//

#if   defined (EMUCFG_CPU_3910)
   #define  MIPS_CLOCK_SPEED              73728000    // 73.728MHz max
   #define  MIPS_SELECTABLE_CLOCK_RATES   4           // in $config:rf
   #define  MIPS_HALTABLE_CPU
#else
   #error "Clock speed not configured"
#endif



// ------------------------------------------------------------------------
//
// Cache parameters
//
// MIPS_CACHE_R3K_STYLE
//    Use R3000 style caches.  These are almost always both direct mapped,
//    and are managed by cache control bits in COP0 $status
//
// MIPS_CACHE_TAGS_VIRTUAL
//    Define this if the cache uses virtual address instead of
//    phys addresses for its tags
//
// MIPS_ICACHE_BYTE_SIZE
//    Total byte size of Insn cache
//
// MIPS_ICACHE_WORDS_PER_LINE
//    Number of words per Insn cache line, per associativity
//
// MIPS_ICACHE_ASSOCIATIVITY
//    Associativity of Insn cache.  1 == direct mapped
//
// MIPS_DCACHE_BYTE_SIZE
//    Total byte size of Data cache
//
// MIPS_DCACHE_WORDS_PER_LINE
//    Number of words per Insn cache line, per associativity
//
// MIPS_DCACHE_ASSOCIATIVITY
//    Associativity of Insn cache.  1 == direct mapped
//
//

#if   defined (EMUCFG_CPU_3910)
   // Caches are physically tagged

   // Primary instruction cache configuration
   // 4K, direct mapped, 1 word per line

   #define  MIPS_ICACHE_BYTE_SIZE         4096
   #define  MIPS_ICACHE_WORDS_PER_LINE    4
   #define  MIPS_ICACHE_ASSOCIATIVITY     1

   // Primary data cache configuration
   // 1K, 2-way set associative, 4 words per line

   #define  MIPS_DCACHE_BYTE_SIZE         1024
   #define  MIPS_DCACHE_WORDS_PER_LINE    1
   #define  MIPS_DCACHE_ASSOCIATIVITY     2
#else
   #error "Need cache config params"
#endif


// Some useful derived values

#define  MIPS_ICACHE_LINES \
   (MIPS_ICACHE_BYTE_SIZE / \
      (MIPS_ICACHE_WORDS_PER_LINE * MIPS_ICACHE_ASSOCIATIVITY * 4) \
   )

#define  MIPS_DCACHE_LINES \
   (MIPS_DCACHE_BYTE_SIZE / \
      (MIPS_DCACHE_WORDS_PER_LINE * MIPS_DCACHE_ASSOCIATIVITY * 4) \
   )


// ------------------------------------------------------------------------
//
// TLB parameters
//
//
// MIPS_TLB_PRESENT
//    Define if the processor has a TLB
//    XXX TLB-less CPUs not implemented yet
//
// MIPS_TLB_ENTRIES
//    Total number of TLB entries
//
// MIPS_TLB_ENTRIES_IS_POW2
//    Define this if number of TLB entries is a power of 2 (lowers runtime
//    checking overheads)
//
// MIPS_TLB_WIRED_ENTRIES
//    Index of first TLB entry that is used with 'write random'.  Entries
//    below this are only modifiable with 'write indexed'.  For R3K, this
//    is typically fixed at 8
//
// MIPS_TLB_WIRED_REGISTER
//    Define this for TLBs that have dynamic 'wired' control with CP0 $wired
//    XXX not implemented yet
//

#if   defined (EMUCFG_CPU_3910)
   #define  MIPS_TLB_PRESENT
   #define  MIPS_TLB_ENTRIES              32
   #define  MIPS_TLB_ENTRIES_IS_POW2
   #define  MIPS_TLB_WIRED_ENTRIES        8
#else
   #error   "Need TLB config params"
#endif


// ------------------------------------------------------------------------
//
// Address spacees
//
// XXX These need to be made ISA-sensitive...?
//

// TLB Mapped, cacheable (based on TLB config)

#define  MIPS_KUSEG_BASE               0x00000000
#define  MIPS_KUSEG_SIZE               0x80000000

// Unmapped, cached

#define  MIPS_KSEG0_BASE               0x80000000
#define  MIPS_KSEG0_SIZE               0x20000000

// Unmapped, uncached

#define  MIPS_KSEG1_BASE               0xa0000000
#define  MIPS_KSEG1_SIZE               0x20000000

// Mapped

#define  MIPS_KSEG2_BASE               0xc0000000
#define  MIPS_KSEG2_SIZE               0x40000000


// Define a mask for extracting physaddr from a KSEGx address

#define  MIPS_KSEGX_PADDR_MASK         0x1fffffff


// Reverse-mapping for direct-addressed phys mem in kseg0, kseg1

#define  MIPS_KSEG0_OR_KSEG1_TO_PHYS(va)  ((va) & MIPS_KSEGX_PADDR_MASK)


// ------------------------------------------------------------------------
//
// Exception handler addresses
//
//

#if   defined (MIPS_TLB_PRESENT)
   // User mode TLB refill address (EXL = 0 only).  For supervisor/ kernel
   // these go via general exception vector

   #define  MIPS_NORM_UTLB_XCPT_ADDR      0x80000000     // kseg0
   #define  MIPS_BOOT_UTLB_XCPT_ADDR      0xbfc00100     // kseg1
#endif


// General exception vector for most purposes

#define  MIPS_NORM_GENERAL_XCPT_ADDR   0x80000080     // kseg0
#define  MIPS_BOOT_GENERAL_XCPT_ADDR   0xbfc00180     // kseg1

// Reset always goes to the same place

#define  MIPS_RESET_XCPT_ADDR          0xbfc00000     // kseg1



#if   defined (EMUCFG_CPU_3910)
   // Special debug exception vector

   #define  MIPS_3910_DEBUG_XCPT_ADDR  0xbfc00200
#endif


// ------------------------------------------------------------------------
//
// Cop0 register 0
//
//

#if   defined (MIPS_TLB_PRESENT)
   #define  MIPS_C0R_INDEX                   0

   // Define its contents

   #if   defined (MIPS_TLB_ENTRIES_IS_POW2)

      #define  MIPS_INDEX_PROBE_FAIL         0x80000000

      #define  MIPS_INDEX_SHIFT              8
      #define  MIPS_INDEX_MASK               \
         ((MIPS_TLB_ENTRIES - 1) << MIPS_INDEX_SHIFT)

      #define  MIPS_INDEX_UNUSED_MASK        \
         (0x7fffffff & ~MIPS_INDEX_MASK)
   #else
      // Not a power of 2 so have to do by hand
      #error "Update TLB index and unused masks"
   #endif
#endif


// ------------------------------------------------------------------------
//
// Cop0 register 1
//
//

#if   defined (MIPS_TLB_PRESENT)
   #define  MIPS_C0R_RANDOM                  1

   // Define its contents

   #if   defined (MIPS_TLB_ENTRIES_IS_POW2)
      #define  MIPS_RANDOM_SHIFT             8
      #define  MIPS_RANDOM_MASK              \
         ((MIPS_TLB_ENTRIES - 1) << MIPS_RANDOM_SHIFT)

      #define  MIPS_RANDOM_UNUSED_MASK       \
         (~MIPS_RANDOM_MASK)
   #else
      // Not a power of 2 so have to do by hand
      #error "Update TLB random and unused masks"
   #endif
#endif


// ------------------------------------------------------------------------
//
// Cop0 register 2
//
//

#if   defined (MIPS_TLB_PRESENT)
   #if   defined (MIPS_32BIT)
      #define  MIPS_C0R_ENTRYLO              2

      // Define its contents

      #define  MIPS_ENTRYLO_UNUSED_MASK      0x000000ff

      #define  MIPS_ENTRYLO_PFN_MASK         0xfffff000
      #define  MIPS_ENTRYLO_PFN_SHIFT        12

      #define  MIPS_ENTRYLO_NOCACHE          0x00000800
      #define  MIPS_ENTRYLO_DIRTY            0x00000400
      #define  MIPS_ENTRYLO_VALID            0x00000200
      #define  MIPS_ENTRYLO_GLOBAL           0x00000100
   #else
      #error "64-bit $entrylo0 not defined"
   #endif
#endif


// ------------------------------------------------------------------------
//
// Cop0 register 3
//
//

#if   defined (EMUCFG_CPU_3910)
   // Custom configuration register

   #define  MIPS_C0R_3910_CONFIG             3

   // Define its contents

   #define  MIPS_3910_CONFIG_UNUSED_MASK     0xffc0f000

   // Insn cache size.  Read only

   #define  MIPS_3910_CONFIG_ICS_MASK        0x00380000
   #define  MIPS_3910_CONFIG_ICS_SHIFT       19

   #define  MIPS_3910_CONFIG_ICS_1K          0x00
   #define  MIPS_3910_CONFIG_ICS_2K          0x01
   #define  MIPS_3910_CONFIG_ICS_4K          0x02
   #define  MIPS_3910_CONFIG_ICS_8K          0x03
   // Other values are reserved


   // Data cache size.  Read only

   #define  MIPS_3910_CONFIG_DCS_MASK        0x00030000
   #define  MIPS_3910_CONFIG_DCS_SHIFT       16

   #define  MIPS_3910_CONFIG_DCS_1K          0x00
   #define  MIPS_3910_CONFIG_DCS_2K          0x01
   #define  MIPS_3910_CONFIG_DCS_4K          0x02
   #define  MIPS_3910_CONFIG_DCS_8K          0x03
   // Other values are reserved


   // Reduced Frequency.  R/W.  00 at reset

   #define  MIPS_3910_CONFIG_RF_MASK         0x00000c00
   #define  MIPS_3910_CONFIG_RF_SHIFT        10

   // Doze mode. R/w. 0 at reset

   #define  MIPS_3910_CONFIG_DOZE            0x00000200

   // Halt mode. R/w. 0 at reset.  1: go to halt state

   #define  MIPS_3910_CONFIG_HALT            0x00000100


   // Lock config. R/w-once.  0 at reset.  Write a 1 to lock config
   // until next reset

   #define  MIPS_3910_CONFIG_LOCK            0x00000080

   // Data Cache burt refill. R/W.  0 at reset. 0 == 4 words.  1== use DRS

   #define  MIPS_3910_CONFIG_DCBR            0x00000040

   // Insn Cache Enable.  R/W.  1 at reset

   #define  MIPS_3910_CONFIG_ICE             0x00000020

   // Data Cache Enable.  R/W.  1 at reset

   #define  MIPS_3910_CONFIG_DCE             0x00000010


   // Insn Cache burst refill size. R/W  00 at reset
   // 00: 4 words, 01: 8 words, 10: 16 words, 11: 32 words

   #define  MIPS_3910_CONFIG_IBRS_MASK       0x0000000c
   #define  MIPS_3910_CONFIG_IBRS_SHIFT      2


   // Data Cache burst refill size. R/W  00 at reset
   // 00: 4 words, 01: 8 words, 10: 16 words, 11: 32 words

   #define  MIPS_3910_CONFIG_DBRS_MASK       0x00000003
   #define  MIPS_3910_CONFIG_DBRS_SHIFT      0


#elif defined (MIPS_64BIT) && defined (MIPS_TLB_PRESENT)
   #error "64-bit $entrylo1 not yet defined"
#endif


// ------------------------------------------------------------------------
//
// Cop0 register 4
//
//

#if   defined (MIPS_TLB_PRESENT)
   #if   defined (MIPS_32BIT)
      #define  MIPS_C0R_CONTEXT                 4

      // Define its contents

      #define  MIPS_CONTEXT_UNUSED_MASK      0x00000003

      #define  MIPS_CONTEXT_PTE_MASK         0xffe00000
      #define  MIPS_CONTEXT_PTE_SHIFT        21

      #define  MIPS_CONTEXT_BAD_VPN_MASK     0x001ffffc
      #define  MIPS_CONTEXT_BAD_VPN_SHIFT    2
   #else
      #error "64-bit $context not defined"
   #endif
#endif


// ------------------------------------------------------------------------
//
// Cop0 register 5
//
//

#if   defined (MIPS_TLB_PRESENT)
   #if   defined (MIPS_64BIT)
      #error "64-bit $pagemask not defined"
   #endif
#endif


// ------------------------------------------------------------------------
//
// Cop0 register 6
//
//

#if   defined (MIPS_TLB_PRESENT)
   #if   defined (MIPS_64BIT)
      #error "64-bit $wired not defined"
   #endif
#endif


// ------------------------------------------------------------------------
//
// Cop0 register 7
//
//

#if   defined (EMUCFG_CPU_3910)
   // Custom cache control register

   #define  MIPS_C0R_3910_CACHE           7

   // Define its contents

   #define  MIPS_3910_CACHE_UNUSED_MASK   0xffffc0ff

   // Cache state is a small stack, like R3K style int stack in $status

   #define  MIPS_3910_CACHE_IALO          0x00002000
   #define  MIPS_3910_CACHE_DALO          0x00001000

   #define  MIPS_3910_CACHE_IALP          0x00000800
   #define  MIPS_3910_CACHE_DALP          0x00000400

   #define  MIPS_3910_CACHE_IALC          0x00000200
   #define  MIPS_3910_CACHE_DALC          0x00000100

#endif


// ------------------------------------------------------------------------
//
// Cop0 register 8
//
//

#define  MIPS_C0R_BADVADDR                8


// ------------------------------------------------------------------------
//
// Cop0 register 9
//
//

#if   defined (MIPS_64BIT)
   #error "64-bit $count not defined"
#endif


// ------------------------------------------------------------------------
//
// Cop0 register 10
//
//

#if   defined (MIPS_TLB_PRESENT)
   #if   defined (MIPS_32BIT)
      #define  MIPS_C0R_ENTRYHI                 10

      // Define its contents

      #define  MIPS_ENTRYHI_UNUSED_MASK      0x0000003f

      #define  MIPS_ENTRYHI_VPN_MASK         0xfffff000
      #define  MIPS_ENTRYHI_VPN_SHIFT        12

      #define  MIPS_ENTRYHI_ASID_MASK        0x00000fc0
      #define  MIPS_ENTRYHI_ASID_SHIFT       6
   #else
      #error "64-bit $entryhi not defined"
   #endif
#endif


// ------------------------------------------------------------------------
//
// Cop0 register 11
//
//

#if   defined (MIPS_64BIT)
   #error "64-bit $compare not defined"
#endif


// ------------------------------------------------------------------------
//
// Cop0 register 12
//
//

#define  MIPS_C0R_STATUS                  12

// Define its contents


// Coprocessor useability control

#define  MIPS_SR_CU_MASK               0xf0000000     // Cop useable mask
#define  MIPS_SR_CU_SHIFT              28

#define  MIPS_SR_CU3                   0x80000000     // Cop3
#define  MIPS_SR_CU2                   0x40000000     // Cop2
#define  MIPS_SR_CU1                   0x20000000     // FP coprocessor
#define  MIPS_SR_CU0                   0x10000000     // System coprocessor

// Unused:                             0x08000000
// Unused:                             0x04000000

// Endian control (set to reverse endian in user mode)

#define  MIPS_SR_REV_ENDIAN            0x02000000


// ---------- Beginning of Diagnostic/ Status fields; highly CPU specific

// Unused:                             0x01000000
// Unused:                             0x00800000

// Bootstrap vectors.  When set, exception base is 0xbfc00000, otherwise
// see MIPS_xxx_INTR_ADDR above

#define  MIPS_SR_BEV                   0x00400000

// Will be set when colliding entries in TLB detected (the TLB is also
// disabled until a hard reset)

#define  MIPS_SR_TLB_SHUTDOWN          0x00200000


#if   defined (MIPS_CACHE_R3K_STYLE)
   // Diag field values specific to R3K style cache management

   // Will be set when a parity ewrror has been detected in the cache
   // (clear by writing a '1' to this bit)

   #define  MIPS_SR_CACHE_PARITY_ERROR    0x00100000

   // Will be set when a data cache miss occurs in test mode (cache isolated)

   #define  MIPS_SR_CACHE_TEST_MISS       0x00080000

   // Controls the zeroing of cache parity bits (0: normal, 1: parity
   // is forced to zero)

   #define  MIPS_SR_CACHE_PARITY_ZERO     0x00040000

   // Control the switching of the data and insn caches (0: normal, 1: swapped)

   #define  MIPS_SR_CACHE_SWAP            0x00020000

   // Controls isolation of cache (0: normal, 1: isolated from bus)

   #define  MIPS_SR_DCACHE_ISOLATE        0x00010000


   // Unused bits in diagnostics field

   #define  MIPS_SR_DIAG_UNUSED           0x01800000

#elif defined (EMUCFG_CPU_3910)
   // Non maskable interrupt status.  Write a 1 to clear it

   #define  MIPS_SR_3910_NMI              0x00100000

   // Unused bits in diagnostics field

   #define  MIPS_SR_DIAG_UNUSED           0x018f0000

#else
   // Unused bits in diagnostics field

   #define  MIPS_SR_DIAG_UNUSED           0x019f0000
#endif

// ---------- End of Diagnostic/ Status fields; highly CPU specific


// Interrupt enable masks

// Combined HW, SW int mask

#define  MIPS_SR_INT_MASK              0x0000ff00
#define  MIPS_SR_INT_SHIFT             8              // Shift from pos 0

// Hardware int mask

#define  MIPS_SR_HWINT_MASK            0x0000fc00
#define  MIPS_SR_HWINT_SHIFT           10             // Shift from pos 0

// Software int mask

#define  MIPS_SR_SWINT_MASK            0x00000300
#define  MIPS_SR_SWINT_SHIFT           6              // Shift from pos 0

// Individual bits

#define  MIPS_SR_INT_HW5               0x00008000     // H/W Interrupt 5
#define  MIPS_SR_INT_HW4               0x00004000     // H/W Interrupt 4
#define  MIPS_SR_INT_HW3               0x00002000     // H/W Interrupt 3
#define  MIPS_SR_INT_HW2               0x00001000     // H/W Interrupt 2
#define  MIPS_SR_INT_HW1               0x00000800     // H/W Interrupt 1
#define  MIPS_SR_INT_HW0               0x00000400     // H/W Interrupt 0
#define  MIPS_SR_INT_SW1               0x00000200     // S/W Interrupt 1
#define  MIPS_SR_INT_SW0               0x00000100     // S/W Interrupt 0


// Unused:                             0x000000c0

// Operating mode - miniature state stack
// Pushing state should move prev, curr to old, prev and clear bottom
// 2 bits.
// Popping state should move old, prev to prev, curr and *LEAVE* old alone

#define  MIPS_SR_MODE_MASK             0x0000003f
#define  MIPS_SR_MODE_PUSH_POP_SHIFT   2

#define  MIPS_SR_OLD_MASK              0x00000030
#define  MIPS_SR_OLD_MODE_USER         0x00000020     // 1:user
#define  MIPS_SR_OLD_INT_ENABLE        0x00000010     // 1:enable

#define  MIPS_SR_PREV_MASK             0x0000000c
#define  MIPS_SR_PREV_MODE_USER        0x00000008     // 1:user
#define  MIPS_SR_PREV_INT_ENABLE       0x00000004     // 1:enable

#define  MIPS_SR_CURR_MASK             0x00000003
#define  MIPS_SR_CURR_MODE_USER        0x00000002     // 1:user
#define  MIPS_SR_CURR_INT_ENABLE       0x00000001     // 1:enable



// Unused bits (Should be zero on write)

#define  MIPS_SR_UNUSED_MASK           (0x0c0000c0 | MIPS_SR_DIAG_UNUSED)



// ------------------------------------------------------------------------
//
// Cop0 register 13
//
//

#define  MIPS_C0R_CAUSE                   13

// Define its contents


// Unused bits (Should be zero on write)

#define  MIPS_CAUSE_UNUSED_MASK        0x4fff0083

// Exception victim is in a branch delay slot

#define  MIPS_CAUSE_BD                 0x80000000

// Coprocessor error - index (0..3) of coprocessor that caused error

#define  MIPS_CAUSE_CE_MASK            0x30000000     // Cop error mask
#define  MIPS_CAUSE_CE_SHIFT           28


// Interrupt status (*not latched*) bits.  Same positions as in $status
// S/W bits retain the last value written

// Hardware int mask

#define  MIPS_CAUSE_HWINT_MASK         0x0000fc00
#define  MIPS_CAUSE_HWINT_SHIFT        10             // Shift from pos 0

// Software int mask

#define  MIPS_CAUSE_SWINT_MASK         0x00000300
#define  MIPS_CAUSE_SWINT_SHIFT        6              // Shift from pos 0

// Combined mask

#define  MIPS_CAUSE_INT_MASK           0x0000ff00
#define  MIPS_CAUSE_INT_SHIFT          8              // Shift from pos 0

#define  MIPS_CAUSE_INT_HW5            0x00008000     // H/W Interrupt 5
#define  MIPS_CAUSE_INT_HW4            0x00004000     // H/W Interrupt 4
#define  MIPS_CAUSE_INT_HW3            0x00002000     // H/W Interrupt 3
#define  MIPS_CAUSE_INT_HW2            0x00001000     // H/W Interrupt 2
#define  MIPS_CAUSE_INT_HW1            0x00000800     // H/W Interrupt 1
#define  MIPS_CAUSE_INT_HW0            0x00000400     // H/W Interrupt 0
#define  MIPS_CAUSE_INT_SW1            0x00000200     // S/W Interrupt 1
#define  MIPS_CAUSE_INT_SW0            0x00000100     // S/W Interrupt 0


// Current exception code (conveniently, its shifted up 2 bits for easy
// jump table indexing)

#define  MIPS_CAUSE_EXC_MASK           0x0000007c
#define  MIPS_CAUSE_EXC_SHIFT          2              // Shift from pos 0

#define  MIPS_CAUSE_EXC_INT            0x00000000     //  0: Interrupt

#if   defined (MIPS_TLB_PRESENT)
   #define  MIPS_CAUSE_EXC_TLB_MOD        0x00000004     //  1: TLB modification
   #define  MIPS_CAUSE_EXC_TLB_LOAD       0x00000008     //  2: TLB miss l/D load
                                                         //     Via TLBREFILL
   #define  MIPS_CAUSE_EXC_TLB_STORE      0x0000000c     //  3: TLB miss D store
                                                         //     Via TLBREFILL
#endif

#define  MIPS_CAUSE_EXC_ADDR_LOAD      0x00000010     //  4: I/D load addr
#define  MIPS_CAUSE_EXC_ADDR_STORE     0x00000014     //  5: D store addr
#define  MIPS_CAUSE_EXC_BUSERR_INSN    0x00000018     //  6: Bus I load
#define  MIPS_CAUSE_EXC_BUSERR_DATA    0x0000001c     //  7: Bus I/D ld/st
#define  MIPS_CAUSE_EXC_SYSCALL        0x00000020     //  8:
#define  MIPS_CAUSE_EXC_BREAK          0x00000024     //  9:
#define  MIPS_CAUSE_EXC_RES_INSN       0x00000028     // 10:
#define  MIPS_CAUSE_EXC_COP_UNUSABLE   0x0000002c     // 11:
#define  MIPS_CAUSE_EXC_OVERFLOW       0x00000030     // 12:

#if   MIPS_ISA >= 2
   #define  MIPS_CAUSE_EXC_TRAP        0x00000034     // 13: Trap exception
#endif
// 12..31 reserved.  Some are used on more advanced processors


// NOTE: We 'borrow' some exc codes for debug hooking.  These will never
//       turn up in $cause, and should *NOT* overlap with real codes.

#if   defined (EMUCFG_CPU_3910)
   #define  EMU_CAUSE_EXC_3910_DEBUG   0x00000007c
#endif

// ------------------------------------------------------------------------
//
// Cop0 register 14
//
//

#define  MIPS_C0R_EPC                     14


// ------------------------------------------------------------------------
//
// Cop0 register 15
//
//

#define  MIPS_C0R_PRID                    15

// Define its contents


#if   defined (EMUCFG_CPU_3910)
   // XXX Revision val to be determined; not in docs
   #define  MIPS_PRID_IMP_VAL          0x22
   #define  MIPS_PRID_REV_VAL          0
#else
   #error "Need PRID config params"
#endif


#define  MIPS_PRID_UNUSED_MASK         0xffff0000

#define  MIPS_PRID_IMP_MASK            0x0000ff00
#define  MIPS_PRID_IMP_SHIFT           8

#define  MIPS_PRID_REV_MASK            0x000000ff
#define  MIPS_PRID_REV_SHIFT           0



// ------------------------------------------------------------------------
//
// Cop0 register 16
//
//

#if   defined (EMUCFG_CPU_3910)
   #define  MIPS_C0R_3910_DEBUG              16

   // Define its contents

   #define  MIPS_3910_DEBUG_DBD        0x80000000     // dbg xcpt in BD slot
   #define  MIPS_3910_DEBUG_DM         0x40000000     // In debug mode
   // Unused                           0x20000000
   // Unused                           0x10000000
   // Unused                           0x0fff0000
   // Unused                           0x00008000
   #define  MIPS_3910_DEBUG_NIS        0x00004000     // NMI status
   #define  MIPS_3910_DEBUG_UMS        0x00002000     // Reserved
   #define  MIPS_3910_DEBUG_OES        0x00001000     // Other xcpt status
   #define  MIPS_3910_DEBUG_TLF        0x00000800     // TLB xcpt
   #define  MIPS_3910_DEBUG_BSF        0x00000400     // Buserr xcpt
   // Unused                           0x00000200
   #define  MIPS_3910_DEBUG_SST        0x00000100     // Single step
   // Unused                           0x00000080
   // Unused                           0x00000040
   #define  MIPS_3910_DEBUG_DINT       0x00000020     // Reserved
   #define  MIPS_3910_DEBUG_DIB        0x00000010     // Reserved
   #define  MIPS_3910_DEBUG_DDBS       0x00000008     // Reserved
   #define  MIPS_3910_DEBUG_DDBL       0x00000004     // Reserved
   #define  MIPS_3910_DEBUG_DBP        0x00000002     // Dbg bp xcpt
   #define  MIPS_3910_DEBUG_DSS        0x00000001     // Dbg step xcpt

   #define  MIPS_3910_DEBUG_MASK       0x3fff82c0
#else
   #if   defined (MIPS_64BIT)
      #error "64-bit $config not defined"
   #endif
#endif


// ------------------------------------------------------------------------
//
// Cop0 register 17
//
//

#if   defined (EMUCFG_CPU_3910)
   #define  MIPS_C0R_3910_DEPC               17

   // Define its contents

#else
   #if   defined (MIPS_64BIT)
      #error "64-bit $lladdr not defined"
   #endif
#endif


// ------------------------------------------------------------------------
//
// Cop0 register 18
//
//

#if   defined (MIPS_64BIT)
   #error "64-bit R4K $watchlo not defined"
#endif


// ------------------------------------------------------------------------
//
// Cop0 register 19
//
//

#if   defined (MIPS_64BIT)
   #error "64-bit R4K $watchhi not defined"
#endif


// ------------------------------------------------------------------------
//
// Cop0 register 20
//
//

#if   defined (MIPS_64BIT)
   #error "64-bit R4K $xcontext not defined"
#endif


// ------------------------------------------------------------------------
//
// Cop0 register 21..25
//
//

// Unused at present


// ------------------------------------------------------------------------
//
// Cop0 register 26
//
//

#if   defined (MIPS_64BIT)
   #error "64-bit R4K $ecc not defined"
#endif


// ------------------------------------------------------------------------
//
// Cop0 register 27
//
//

#if   defined (MIPS_64BIT)
   #error "64-bit R4K $cacherr not defined"
#endif


// ------------------------------------------------------------------------
//
// Cop0 register 28
//
//

#if   defined (MIPS_64BIT)
   #error "64-bit R4K $taglo not defined"
#endif


// ------------------------------------------------------------------------
//
// Cop0 register 29
//
//

#if   defined (MIPS_64BIT)
   #error "64-bit R4K $taghi not defined"
#endif


// ------------------------------------------------------------------------
//
// Cop0 register 30
//
//

#if   defined (MIPS_64BIT)
   #error "64-bit R4K $errorepc not defined"
#endif


// ------------------------------------------------------------------------
//
// Cop0 register 31
//
//

// Unused at present


// ------------------------------------------------------------------------
//
// GPRs of interest
//
// NOTE: These are ISA and ABI sensitive.  Emulator currently only relies
//       on $zero and $ra; the rest are here for convenience/ doc purposes
//       Others may be used to specialise instruction emulation for 'popular'
//       register values, so its worth defining them
//

#if   defined (MIPS_32BIT)
   #define  MIPS_GPR_ZERO  0     // wired zero
   #define  MIPS_GPR_AT    1     // assembler temp
   #define  MIPS_GPR_V0    2     // return value
   #define  MIPS_GPR_V1    3
   #define  MIPS_GPR_A0    4     // argument registers
   #define  MIPS_GPR_A1    5
   #define  MIPS_GPR_A2    6
   #define  MIPS_GPR_A3    7
   #define  MIPS_GPR_T0    8     // caller saved
   #define  MIPS_GPR_T1    9
   #define  MIPS_GPR_T2    10
   #define  MIPS_GPR_T3    11

   #define  MIPS_GPR_T4    12    // caller saved - 32 bit
   #define  MIPS_GPR_TA0   12    // caller saved in 32 bit

   #define  MIPS_GPR_T5    13
   #define  MIPS_GPR_TA1   13

   #define  MIPS_GPR_T6    14
   #define  MIPS_GPR_TA2   14

   #define  MIPS_GPR_T7    15
   #define  MIPS_GPR_TA3   15

   #define  MIPS_GPR_S0    16    // callee saved
   #define  MIPS_GPR_S1    17
   #define  MIPS_GPR_S2    18
   #define  MIPS_GPR_S3    19
   #define  MIPS_GPR_S4    20
   #define  MIPS_GPR_S5    21
   #define  MIPS_GPR_S6    22
   #define  MIPS_GPR_S7    23
   #define  MIPS_GPR_T8    24    // code generator

   #define  MIPS_GPR_T9    25
   #define  MIPS_GPR_JP    25    // PIC jump register

   #define  MIPS_GPR_K0    26    // kernel temporary
   #define  MIPS_GPR_K1    27
   #define  MIPS_GPR_GP    28    // global pointer
   #define  MIPS_GPR_SP    29    // stack pointer

   #define  MIPS_GPR_FP    30    // frame pointer
   #define  MIPS_GPR_S8    30    // calle saved

   #define  MIPS_GPR_RA    31    // return address
#else
   // XXX Define 64-bit ABI assignments?

   #define  MIPS_GPR_ZERO  0     // wired zero
   #define  MIPS_GPR_RA    31    // return address
#endif


#endif

