// ========================================================================
//
// pr31700.h
//
// Definitions for the chip-specific I/O registers on the PR31700.
// Page/ chapter references are for the PR31700 Users Guide [PR31700UG]
//
// The Reset () methods will set the regs as per a hard reset of the
// physical device
//
// NOTE: Values in these unions are assumed to be in host-endian order
//       to match the bitfield ordering
//
// NOTE: Reserved fields should be named for the lowest bit of the reserved
//       field
//
//
// Copyright (c) 1999 Frank Somers, Denis Hennessy,
// Ground Zero Development <frank@g0dev.com>, <denis@g0dev.com>
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation; either version 2 of the License, or (at your
// option) any later version.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// ========================================================================

#ifndef  PR31700_H
#define  PR31700_H

#include <basetype.h>


// ------------------------------------------------------------------------
//
// Physical address space map for pr31700-based devices
// Note that several devices are aliased, and that the physical map is
// different in user and kernel modes.  Any device above 512Mb requires
// a TLB mapping to reach.
//
//
// Size     Address     Segment     Devices
// =======  =========   ==========  ==========
// 16 Mb    FF00_0000   reserved    reserved
// 1 Gb     C000_0000   kseg2       reserved
// 1 GB     7C00_0000   kuser       reserved
// 64 Mb    7800_0000   kuser       MCS3
// 64 Mb    7400_0000   kuser       MCS2
// 64 Mb    7000_0000   kuser       MCS1
// 64 Mb    6C00_0000   kuser       MCS0
// 64 Mb    6800_0000   kuser       Card 2 (Memory)
// 64 Mb    6400_0000   kuser       Card 1 (Memory)
// 64 Mb    6000_0000   kuser       CS3 (if enabled)
// 64 Mb    5C00_0000   kuser       CS2 (if enabled)
// 64 Mb    5800_0000   kuser       CS1 (if enabled)
// 128 Mb   5000_0000   kuser       CS0 (ROM)
// 128 Mb   4800_0000   kuser       reserved
// 32 Mb    4600_0000   kuser       DRAM BANK 1
// 32 Mb    4400_0000   kuser       DRAM BANK 0
// 32 Mb    4200_0000   kuser       CS1 (if enabled, else DRAM BANK 0)
// 32 Mb    4000_0000   kuser       CS1 (if enabled, else DRAM BANK 1)
// 512 Mb   2000_0000   reserved    reserved
// 240 Mb   1100_0000   kseg0,kseg1 CS0 (ROM)
// 1 Mb     10F0_0000   kseg0,kseg1 SDRAM BANK 1 Mode Register
// 1 Mb     10E0_0000   kseg0,kseg1 SDRAM BANK 0 Mode Register
// 2 Mb     10C0_0000   kseg0,kseg1 PR31700 Registers
// 4 Mb     1080_0000   kseg0,kseg1 CS3
// 4 Mb     1040_0000   kseg0,kseg1 CS2
// 4 Mb     1000_0000   kseg0,kseg1 CS1
// 64 Mb    0C00_0000   kseg0,kseg1 Card 2 (I/O or Attribute)
// 64 Mb    0800_0000   kseg0,kseg1 Card 1 (I/O or Attribute)
// 32 Mb    0600_0000   kseg0,kseg1 DRAM BANK 1
// 32 Mb    0400_0000   kseg0,kseg1 DRAM BANK 0
// 32 Mb    0200_0000   kseg0,kseg1 CS1 (if enabled, else DRAM BANK 1)
// 32 Mb    0000_0000   kseg0,kseg1 CS1 (if enabled, else DRAM BANK 0)
//
//


// ------------------------------------------------------------------------
//
// Offsets for the PR31700 Registers from physad 0x10c00000
//
// Offset   R/W   Desc
// ======   ===   ====
//
// 000      R/W   Memory Configuration 0
// 004      R/W   Memory Configuration 1
// 008      R/W   Memory Configuration 2
// 00C      R/W   Memory Configuration 3
// 010      R/W   Memory Configuration 4
// 014      W     Memory Configuration 5
// 018      W     Memory Configuration 6
// 01C      W     Memory Configuration 7
// 020      W     Memory Configuration 8
// 024      ---
// 028      R/W   Video Control 1
// 02C      W     Video Control 2
// 030      W     Video Control 3
// 034      W     Video Control 4
// 038      W     Video Control 5
// 03C      W     Video Control 6
// 040      W     Video Control 7
// 044      W     Video Control 8
// 048      W     Video Control 9
// 04C      W     Video Control 10
// 050      W     Video Control 11
// 054      W     Video Control 12
// 058      W     Video Control 13
// 05C      W     Video Control 14
// 060      W     SIB Size
// 064      W     SIB Sound Receive Start
// 068      W     SIB Sound Transmit Start
// 06C      W     SIB Tel Receive Start
// 070      W     SIB Tel Transmit Start
// 074      R/W   SIB Control
// 078      R/W   SIB Sound Holding Register
// 07C      R/W   SIB Tel Holding Register
// 080      R/W   SIB SF0 Control
// 084      R/W   SIB SF1 Control
// 088      R     SIB SF0 Status
// 08C      R     SIB SF1 Status
// 090      R/W   SIB DMA Control
// 094      ---
// 098      ---
// 09C      ---
// 0A0      R/W   IR Control 1
// 0A4      W     IR Control 2
// 0A8      W     IR Holding Register
// 0AC      ---
// 0B0      R/W   UARTA Control 1
// 0B4      W     UARTA Control 2
// 0B8      W     UARTA DMA Control 1
// 0BC      W     UARTA DMA Control 2
// 0C0      R     UARTA DMA Count
// 0C4      R/W   UARTA Holding Register
// 0C8      R/W   UARTB Control 1
// 0CC      W     UARTB Control 2
// 0D0      W     UARTB DMA Control 1
// 0D4      W     UARTB DMA Control 2
// 0D8      R     UARTB DMA Count
// 0DC      R/W   UARTB Holding Register
// 0E0      R/W   Reserved (Undocumented MBUS reg)
// 0E4      W     Reserved (Undocumented MBUS reg)
// 0E8      W     Reserved (Undocumented MBUS reg)
// 0EC      W     Reserved (Undocumented MBUS reg)
// 0F0      R     Reserved (Undocumented MBUS reg)
// 0F4      W     Reserved
// 0F8      R/W   Reserved
// 0FC      ---
// 100      W     Clear Interrupt 1
// 100      R     Interrupt Status 1
// 104      W     Clear Interrupt 2
// 104      R     Interrupt Status 2
// 108      W     Clear Interrupt 3
// 108      R     Interrupt Status 3
// 10C      W     Clear Interrupt 4
// 10C      R     Interrupt Status 4
// 110      W     Clear Interrupt 5
// 110      R     Interrupt Status 5
// 114      W     Clear Interrupt 6    ??? not in manual but looks right
// 114      R     Interrupt Status 6
// 118      R/W   Enable Interrupt 1
// 11C      R/W   Enable Interrupt 2
// 120      R/W   Enable Interrupt 3
// 124      R/W   Enable Interrupt 4
// 128      R/W   Enable Interrupt 5
// 12C      R/W   Enable Interrupt 6
// 130      ---
// 134      ---
// 138      ---
// 13C      ---
// 140      R     RTC High
// 144      R     RTC Low
// 148      R/W   Alarm High
// 14C      R/W   Alarm Low
// 150      R/W   Timer Control
// 154      R/W   Periodic Timer
// 158      ---
// 15C      ---
// 160      R/W   SPI Control
// 164      R/W   SPI Holding Register
// 168      ---
// 16C      ---
// 170      ---
// 174      ---
// 178      ---
// 17C      ---
// 180      R/W   IO Control
// 184      R/W   Multi-function IO Data Out
// 188      R/W   Multi-function IO Direction
// 18C      R     Multi-function IO Data In
// 190      R/W   Multi-function IO Select
// 194      R/W   IO Power Down
// 198      R/W   Multi-function IO Power Down
// 19C      ---
// 1A0      ---
// 1A4      ---
// 1A8      ---
// 1AC      ---
// 1B0      ---
// 1B4      ---
// 1B8      ---
// 1BC      ---
// 1C0      R/W   Clock Control
// 1C4      R/W   Power Control
// 1C8      R/W   SIU Test
// 1CC      ---
// 1D0      ---
// 1D4      ---
// 1D8      R/W   CHI Control
// 1DC      R/W   CHI Pointer Enable
// 1E0      W     CHI Receive Pointer A
// 1E4      W     CHI Receive Pointer B
// 1E8      W     CHI Transmit Pointer A
// 1EC      W     CHI Transmit Pointer B
// 1F0      R/W   CHI Size
// 1F4      W     CHI Receive Start
// 1F8      W     CHI Transmit Start
// 1FC      R/W   CHI Holding Register
// 


// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
//
// BIU Registers
//
//

// ------------------------------------------------------------------------
//
// BIU Memory Configuration 0 Register (section 4.7.1)
// All bits read/ write
//
// NOTE: Reset value for cs0size dependent on external TESTSIU pin
//       Currently defaulting to 32-bit port
//
// Fields of interest:
//    enrmap2
//    enrmap1
//    encs3user
//    encs2user
//    encs1user
//    encs1dram
//

union BiuMcr0 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 cs0size          : 1;
         u32 cs1size          : 1;
         u32 cs2size          : 1;
         u32 cs3size          : 1;
         u32 colsel0          : 4;
         u32 colsel1          : 4;
         u32 rowsel0          : 2;
         u32 rowsel1          : 2;
         u32 bank0conf        : 2;
         u32 bank1conf        : 2;
         u32 encs1dram        : 1;
         u32 encs1user        : 1;
         u32 encs2user        : 1;
         u32 encs3user        : 1;
         u32 enwrinpage       : 1;
         u32 enrmap1          : 1;
         u32 enrmap2          : 1;
         u32 showpr31700      : 1;
         u32 ensdrampd        : 1;
         u32 disdqminit       : 1;
         u32 endclkouttri     : 1;
         u32 reserved31       : 1;
#else
         u32 reserved31       : 1;
         u32 endclkouttri     : 1;
         u32 disdqminit       : 1;
         u32 ensdrampd        : 1;
         u32 showpr31700      : 1;
         u32 enrmap2          : 1;
         u32 enrmap1          : 1;
         u32 enwrinpage       : 1;
         u32 encs3user        : 1;
         u32 encs2user        : 1;
         u32 encs1user        : 1;
         u32 encs1dram        : 1;
         u32 bank1conf        : 2;
         u32 bank0conf        : 2;
         u32 rowsel1          : 2;
         u32 rowsel0          : 2;
         u32 colsel1          : 4;
         u32 colsel0          : 4;
         u32 cs3size          : 1;
         u32 cs2size          : 1;
         u32 cs1size          : 1;
         u32 cs0size          : 1;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000001;
      }
};


// ------------------------------------------------------------------------
//
// BIU Memory Configuration 1 Register (section 4.7.2)
// All bits read/ write
//
//

union BiuMcr1 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 mcs0accval2      : 4;
         u32 mcs0accval1      : 4;
         u32 mcs1accval2      : 4;
         u32 mcs1accval1      : 4;
         u32 mcs2accval2      : 4;
         u32 mcs2accval1      : 4;
         u32 mcs3accval2      : 4;
         u32 mcs3accval1      : 4;
#else
         u32 mcs3accval1      : 4;
         u32 mcs3accval2      : 4;
         u32 mcs2accval1      : 4;
         u32 mcs2accval2      : 4;
         u32 mcs1accval1      : 4;
         u32 mcs1accval2      : 4;
         u32 mcs0accval1      : 4;
         u32 mcs0accval2      : 4;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// BIU Memory Configuration 2 Register (section 4.7.3)
// All bits read/ write
//
//

union BiuMcr2 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 cs0accval2       : 4;
         u32 cs0accval1       : 4;
         u32 cs1accval2       : 4;
         u32 cs1accval1       : 4;
         u32 cs2accval2       : 4;
         u32 cs2accval1       : 4;
         u32 cs3accval2       : 4;
         u32 cs3accval1       : 4;
#else
         u32 cs3accval1       : 4;
         u32 cs3accval2       : 4;
         u32 cs2accval1       : 4;
         u32 cs2accval2       : 4;
         u32 cs1accval1       : 4;
         u32 cs1accval2       : 4;
         u32 cs0accval1       : 4;
         u32 cs0accval2       : 4;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x000000ff;
      }
};


// ------------------------------------------------------------------------
//
// BIU Memory Configuration 3 Register (section 4.7.4)
// All bits read/ write
//
// Fields of interest:
//    card2ioen
//    card1ioen
//

union BiuMcr3 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 reserved0        : 3;
         u32 _8portsel        : 1;
         u32 card1ioen        : 1;
         u32 card2ioen        : 1;
         u32 card1waiten      : 1;
         u32 card2waiten      : 1;
         u32 encs0page        : 1;
         u32 encs1page        : 1;
         u32 encs2page        : 1;
         u32 encs3page        : 1;
         u32 enmcs0page       : 1;
         u32 enmcs1page       : 1;
         u32 enmcs2page       : 1;
         u32 enmcs3page       : 1;
         u32 card1ioaccval    : 4;
         u32 card2ioaccval    : 4;
         u32 card1accval      : 4;
         u32 card2accval      : 4;
#else
         u32 card2accval      : 4;
         u32 card1accval      : 4;
         u32 card2ioaccval    : 4;
         u32 card1ioaccval    : 4;
         u32 enmcs3page       : 1;
         u32 enmcs2page       : 1;
         u32 enmcs1page       : 1;
         u32 enmcs0page       : 1;
         u32 encs3page        : 1;
         u32 encs2page        : 1;
         u32 encs1page        : 1;
         u32 encs0page        : 1;
         u32 card2waiten      : 1;
         u32 card1waiten      : 1;
         u32 card2ioen        : 1;
         u32 card1ioen        : 1;
         u32 _8portsel        : 1;
         u32 reserved0        : 3;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// BIU Memory Configuration 4 Register (section 4.7.5)
// All bits read/ write
//
// Fields of interest:
//    clrwrbuserrint
//    enwatch
//

union BiuMcr4 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 rfshval0         : 6;
         u32 reserved6        : 2;
         u32 rfshval1         : 6;
         u32 enrfsh0          : 1;
         u32 enrfsh1          : 1;
         u32 mempowerdown     : 1;
         u32 reserved17       : 3;
         u32 watchtimeval     : 4;
         u32 enwatch          : 1;
         u32 enbank0opt       : 1;
         u32 enbank1opt       : 1;
         u32 clrwrbuserrint   : 1;
         u32 dissnoop         : 1;
         u32 enarb            : 1;
         u32 enedobank0       : 1;
         u32 enedobank1       : 1;
#else
         u32 enedobank1       : 1;
         u32 enedobank0       : 1;
         u32 enarb            : 1;
         u32 dissnoop         : 1;
         u32 clrwrbuserrint   : 1;
         u32 enbank1opt       : 1;
         u32 enbank0opt       : 1;
         u32 enwatch          : 1;
         u32 watchtimeval     : 4;
         u32 reserved17       : 3;
         u32 mempowerdown     : 1;
         u32 enrfsh1          : 1;
         u32 enrfsh0          : 1;
         u32 rfshval1         : 6;
         u32 reserved6        : 2;
         u32 rfshval0         : 6;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00010000;
      }
};


// ------------------------------------------------------------------------
//
// BIU Memory Configuration 5 Register (section 4.7.6)
// All bits WRITE-ONLY
//
// Fields of interest:
//    startval2
//    mask2
//

union BiuMcr5 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 mask2            : 4;
         u32 reserved4        : 5;
         u32 startval2        : 23;
#else
         u32 startval2        : 23;
         u32 reserved4        : 5;
         u32 mask2            : 4;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// BIU Memory Configuration 6 Register (section 4.7.7)
// All bits WRITE-ONLY
//
// Fields of interest:
//    startval1
//    mask1
//

union BiuMcr6 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 mask1            : 4;
         u32 reserved4        : 5;
         u32 startval1        : 23;
#else
         u32 startval1        : 23;
         u32 reserved4        : 5;
         u32 mask1            : 4;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// BIU Memory Configuration 7 Register (section 4.7.8)
// All bits WRITE-ONLY
//
// Fields of interest:
//    rmapadd2
//

union BiuMcr7 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 reserved0        : 9;
         u32 rmapadd2         : 23;
#else
         u32 rmapadd2         : 23;
         u32 reserved0        : 9;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// BIU Memory Configuration 8 Register (section 4.7.9)
// All bits WRITE-ONLY
//
// Fields of interest:
//    rmapadd2
//

union BiuMcr8 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 reserved0        : 9;
         u32 rmapadd1         : 23;
#else
         u32 rmapadd1         : 23;
         u32 reserved0        : 9;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Combined BIU registers
//
//

struct BiuRegisters {
   public:
      BiuMcr0 mcr0;
      BiuMcr1 mcr1;
      BiuMcr2 mcr2;
      BiuMcr3 mcr3;
      BiuMcr4 mcr4;
      BiuMcr5 mcr5;
      BiuMcr6 mcr6;
      BiuMcr7 mcr7;
      BiuMcr8 mcr8;

   public:
      void Reset (void) {
         mcr0.Reset ();
         mcr1.Reset ();
         mcr2.Reset ();
         mcr3.Reset ();
         mcr4.Reset ();
         mcr5.Reset ();
         mcr6.Reset ();
         mcr7.Reset ();
         mcr8.Reset ();
      }
};


// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
//
// Video control Registers
//
//

// ------------------------------------------------------------------------
//
// Video Control 1 Register (section 16.4.1)
// All bits read/ write apart from linecnt
//
//

union Vidcr1 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 envid            : 1;
         u32 dispon           : 1;
         u32 invvid           : 1;
         u32 dfmode           : 1;
         u32 disp8            : 1;
         u32 dispsplit        : 1;
         u32 bitsel           : 2;
         u32 enfreezeframe    : 1;
         u32 viddoneval       : 7;
         u32 baudval          : 5;
         u32 loaddly          : 1;
         u32 linecnt          : 10;
#else
         u32 linecnt          : 10;
         u32 loaddly          : 1;
         u32 baudval          : 5;
         u32 viddoneval       : 7;
         u32 enfreezeframe    : 1;
         u32 bitsel           : 2;
         u32 dispsplit        : 1;
         u32 disp8            : 1;
         u32 dfmode           : 1;
         u32 invvid           : 1;
         u32 dispon           : 1;
         u32 envid            : 1;

#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Video Control 2 Register (section 16.4.2)
// Write only.
//
//

union Vidcr2 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 lineval          : 10;
         u32 reserved10       : 2;
         u32 horzval          : 9;
         u32 reserved21       : 1;
         u32 vidrate          : 10;

#else
         u32 vidrate          : 10;
         u32 reserved21       : 1;
         u32 horzval          : 9;
         u32 reserved10       : 2;
         u32 lineval          : 10;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Video Control 3 Register (section 16.4.3)
// Write only.
//
//

union Vidcr3 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 reserved0        : 4;
         u32 vidbasehi        : 16;
         u32 vidbank          : 12;
#else
         u32 vidbank          : 12;
         u32 vidbasehi        : 16;
         u32 reserved0        : 4;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Video Control 4 Register (section 16.4.4)
// Write only.
//
//

union Vidcr4 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 reserved0        : 4;
         u32 vidbaselo        : 16;
         u32 framemaskval     : 4;
         u32 dfval            : 8;
#else
         u32 dfval            : 8;
         u32 framemaskval     : 4;
         u32 vidbaselo        : 16;
         u32 reserved0        : 4;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Video Control 5 Register (section 16.4.5)
// Write only.
//
//

union Vidcr5 {
   public:
      u32   value;
      struct {
         u32   redsel         : 32;
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Video Control 6 Register (section 16.4.6)
// Write only.
//
//

union Vidcr6 {
   public:
      u32   value;
      struct {
         u32   greensel       : 32;
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Video Control 7 Register (section 16.4.7)
// Write only.
//
//

union Vidcr7 {
   public:
      u32   value;
      struct {
         u32   bluesel        : 32;
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Video Control 8 Register (section 16.4.8)
// Write only.
//
//

union Vidcr8 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 pat2_3           : 12;
         u32 reserved12       : 20;
#else
         u32 reserved12       : 20;
         u32 pat2_3           : 12;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Video Control 9 Register (section 16.4.9)
// Write only.
//
//

union Vidcr9 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 pat2_4           : 16;
         u32 pat3_4           : 16;
#else
         u32 pat3_4           : 16;
         u32 pat2_4           : 16;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Video Control 10 Register (section 16.4.10)
// Write only.
//
//

union Vidcr10 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 pat4_5           : 20;
         u32 reserved20       : 12;
#else
         u32 reserved20       : 12;
         u32 pat4_5           : 20;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Video Control 11 Register (section 16.4.11)
// Write only.
//
//

union Vidcr11 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 pat3_5           : 20;
         u32 reserved20       : 12;
#else
         u32 reserved20       : 12;
         u32 pat3_5           : 20;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Video Control 12 Register (section 16.4.12)
// Write only.
//
//

union Vidcr12 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 pat6_7           : 28;
         u32 reserved28       : 4;
#else
         u32 reserved28       : 4;
         u32 pat6_7           : 28;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Video Control 13 Register (section 16.4.13)
// Write only.
//
//

union Vidcr13 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 pat5_7           : 28;
         u32 reserved28       : 4;
#else
         u32 reserved28       : 4;
         u32 pat5_7           : 28;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Video Control 14 Register (section 16.4.14)
// Write only.
//
//

union Vidcr14 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 pat4_7           : 28;
         u32 reserved28       : 4;
#else
         u32 reserved28       : 4;
         u32 pat4_7           : 28;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Combined Video registers
//
//

struct VidRegisters {
   public:
      Vidcr1 vcr1;
      Vidcr2 vcr2;
      Vidcr3 vcr3;
      Vidcr4 vcr4;
      Vidcr5 vcr5;
      Vidcr6 vcr6;
      Vidcr7 vcr7;
      Vidcr8 vcr8;
      Vidcr9 vcr9;
      Vidcr10 vcr10;
      Vidcr11 vcr11;
      Vidcr12 vcr12;
      Vidcr13 vcr13;
      Vidcr14 vcr14;

   public:
      void Reset (void) {
         vcr1.Reset ();
         vcr2.Reset ();
         vcr3.Reset ();
         vcr4.Reset ();
         vcr5.Reset ();
         vcr6.Reset ();
         vcr7.Reset ();
         vcr8.Reset ();
         vcr9.Reset ();
         vcr10.Reset ();
         vcr11.Reset ();
         vcr12.Reset ();
         vcr13.Reset ();
         vcr14.Reset ();
      }
};


// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
//
// SIB Registers
//
//

// ------------------------------------------------------------------------
//
// SIB Size Register (section 12.4.1)
// Write-only
//
//

union Sibsiz {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 reserved0        : 2;
         u32 telsize          : 12;
         u32 reserved14       : 4;
         u32 sndsize          : 12;
         u32 reserved30       : 2;
#else
         u32 reserved30       : 2;
         u32 sndsize          : 12;
         u32 reserved14       : 4;
         u32 telsize          : 12;
         u32 reserved0        : 2;

#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// SIB Sound RX Start Register (section 12.4.2)
// Write-only
//
//

union Sibsndrxst {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 reserved0        : 2;
         u32 sndrxstart       : 30;
#else
         u32 sndrxstart       : 30;
         u32 reserved0        : 2;

#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// SIB Sound TX Start Register (section 12.4.3)
// Write-only
//
//

union Sibsndtxst {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 reserved0        : 2;
         u32 sndtxstart       : 30;
#else
         u32 sndtxstart       : 30;
         u32 reserved0        : 2;

#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// SIB Telecom RX Start Register (section 12.4.4)
// Write-only
//
//

union Sibtelrxst {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 reserved0        : 2;
         u32 telrxstart       : 30;
#else
         u32 telrxstart       : 30;
         u32 reserved0        : 2;

#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// SIB Telecom TX Start Register (section 12.4.5)
// Write-only
//
//

union Sibteltxst {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 reserved0        : 2;
         u32 teltxstart       : 30;
#else
         u32 teltxstart       : 30;
         u32 reserved0        : 2;

#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// SIB Control Register (section 12.4.6)
// Read/Write
//
//

union Sibctrl {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 ensib            : 1;
         u32 ensf0            : 1;
         u32 ensf1            : 1;
         u32 sibloop          : 1;
         u32 ensnd            : 1;
         u32 entel            : 1;
         u32 selsndsf1        : 1;
         u32 seltelsf1        : 1;
         u32 sndfsdiv         : 7;
         u32 snd16            : 1;
         u32 telfsdiv         : 7;
         u32 tel16            : 1;
         u32 sibsclkdiv       : 3;
         u32 rmonosndin       : 1;
         u32 sndmono          : 1;
         u32 endmatest        : 1;
         u32 encnttest        : 1;
         u32 sibirq           : 1;
#else
         u32 sibirq           : 1;
         u32 encnttest        : 1;
         u32 endmatest        : 1;
         u32 sndmono          : 1;
         u32 rmonosndin       : 1;
         u32 sibsclkdiv       : 3;
         u32 tel16            : 1;
         u32 telfsdiv         : 7;
         u32 snd16            : 1;
         u32 sndfsdiv         : 7;
         u32 seltelsf1        : 1;
         u32 selsndsf1        : 1;
         u32 entel            : 1;
         u32 ensnd            : 1;
         u32 sibloop          : 1;
         u32 ensf1            : 1;
         u32 ensf0            : 1;
         u32 ensib            : 1;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// SIB Sound TX Holding Register (section 12.4.7)
// Write-only
//
//

union Sibsndtxhld {
   public:
      u32   value;
      struct {
         u32 sndtxhold        : 32;
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// SIB Sound RX Holding Register (section 12.4.8)
// Read-only
//
//

union Sibsndrxhld {
   public:
      u32   value;
      struct {
         u32 sndrxhold        : 32;
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// SIB Telecom TX Holding Register (section 12.4.9)
// Write-only
//
//

union Sibteltxhld {
   public:
      u32   value;
      struct {
         u32 teltxhold        : 32;
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// SIB Telecom RX Holding Register (section 12.4.10)
// Read-only
//
//

union Sibtelrxhld {
   public:
      u32   value;
      struct {
         u32 telrxhold        : 32;
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// SIB Subframe 0 Control Register (section 12.4.11)
// Read-Write
//
//

union Sibsf0cr {
   public:
      u32   value;
      struct {
         u32 sf0aux           : 32;
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// SIB Subframe 1 Control Register (section 12.4.12)
// Read-Write
//
//

union Sibsf1cr {
   public:
      u32   value;
      struct {
         u32 sf1aux           : 32;
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// SIB Subframe 0 Status Register (section 12.4.13)
// Read-Write
//
//

union Sibsf0sr {
   public:
      u32   value;
      struct {
         u32 sf0stat          : 32;
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// SIB Subframe 1 Status Register (section 12.4.14)
// Read-Write
//
//

union Sibsf1sr {
   public:
      u32   value;
      struct {
         u32 sf1stat          : 32;
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// SIB DMA Control Register (section 12.4.15)
// Read-Write
//
//

union Sibdmacr {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 endmatxtel       : 1;
         u32 endmarxtel       : 1;
         u32 teldmaptr        : 12;
         u32 teldmaloop       : 1;
         u32 telbuff1time     : 1;
         u32 endmatxsnd       : 1;
         u32 endmarxsnd       : 1;
         u32 snddmaptr        : 12;
         u32 snddmaloop       : 1;
         u32 sndbuff1time     : 1;
#else
         u32 sndbuff1time     : 1;
         u32 snddmaloop       : 1;
         u32 snddmaptr        : 12;
         u32 endmarxsnd       : 1;
         u32 endmatxsnd       : 1;
         u32 telbuff1time     : 1;
         u32 teldmaloop       : 1;
         u32 teldmaptr        : 12;
         u32 endmarxtel       : 1;
         u32 endmatxtel       : 1;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Combined SIB registers
//
//

struct SibRegisters {
   public:
      Sibsiz siz;
      Sibsndrxst sndrxst;
      Sibsndtxst sndtxst;
      Sibtelrxst telrxst;
      Sibteltxst teltxst;
      Sibctrl ctrl;
      Sibsndtxhld sndtxhld;
      Sibsndrxhld sndrxhld;
      Sibteltxhld teltxhld;
      Sibtelrxhld telrxhld;
      Sibsf0cr sf0cr;
      Sibsf1cr sf1cr;
      Sibsf0sr sf0sr;
      Sibsf1sr sf1sr;
      Sibdmacr dmacr;

   public:
      void Reset (void) {
         siz.Reset ();
         sndrxst.Reset ();
         sndtxst.Reset ();
         telrxst.Reset ();
         teltxst.Reset ();
         ctrl.Reset ();
         sndtxhld.Reset ();
         sndrxhld.Reset ();
         teltxhld.Reset ();
         telrxhld.Reset ();
         sf0cr.Reset ();
         sf1cr.Reset ();
         sf0sr.Reset ();
         sf1sr.Reset ();
         dmacr.Reset ();
      }
};


// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
//
// Power control registers
//
//

// ------------------------------------------------------------------------
//
// Power Control Register (section 11.3.1)
// Read-Write
//
//

union Powcr {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 vccon            : 1;
         u32 pwrcs            : 1;
         u32 coldstart        : 1;
         u32 dbnconbtn        : 1;
         u32 stopcpu          : 1;
         u32 bpdbvcc3         : 1;
         u32 reserved6        : 1;
         u32 selc2ms          : 1;
         u32 forceshutdwnocc  : 1;
         u32 forceshutdwn     : 1;
         u32 enforceshutdwn   : 1;
         u32 enstptimer       : 1;
         u32 stptimerval      : 4;
         u32 reserved16       : 9;
         u32 divmod           : 1;
         u32 slowbus          : 1;
         u32 vidrf            : 2;
         u32 pwrok            : 1;
         u32 pwrint           : 1;
         u32 onbutn           : 1;
#else
         u32 onbutn           : 1;
         u32 pwrint           : 1;
         u32 pwrok            : 1;
         u32 vidrf            : 2;
         u32 slowbus          : 1;
         u32 divmod           : 1;
         u32 reserved16       : 9;
         u32 stptimerval      : 4;
         u32 enstptimer       : 1;
         u32 enforceshutdwn   : 1;
         u32 forceshutdwn     : 1;
         u32 forceshutdwnocc  : 1;
         u32 selc2ms          : 1;
         u32 reserved6        : 1;
         u32 bpdbvcc3         : 1;
         u32 stopcpu          : 1;
         u32 dbnconbtn        : 1;
         u32 coldstart        : 1;
         u32 pwrcs            : 1;
         u32 vccon            : 1;
#endif
      } bits;

   public:
      void Reset (void) {
#if   1
         // Rigged to simulate power on immediately when started

         value = 0;
         bits.onbutn = 1;
         bits.pwrok = 1;
         bits.coldstart = 1;
         bits.pwrcs = 1;
         bits.vccon = 1;
#else
         value = 0xA0000004;
#endif
      }
};


// ------------------------------------------------------------------------
//
// Combined Power registers
//
//

struct PowRegisters {
   public:
      Powcr powcr;

   public:
      void Reset (void) {
         powcr.Reset ();
      }
};


// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
//
// UART control registers
//
//

// ------------------------------------------------------------------------
//
// UART Control 1 Register (section 15.5.1)
// Read-Write
//
//

union Uartcr1 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL

         u32 enuart           : 1;
         u32 enparity         : 1;
         u32 evenparity       : 1;
         u32 bit_7            : 1;
         u32 loopback         : 1;
         u32 twostop          : 1;
         u32 distxd           : 1;
         u32 dtinvert         : 1;
         u32 pulseopt1        : 1;
         u32 pulseopt2        : 1;
         u32 endmaloop        : 1;
         u32 endmatest        : 1;
         u32 enbreakhalt      : 1;
         u32 testmode         : 1;
         u32 endmatx          : 1;
         u32 endmarx          : 1;
         u32 reserved16       : 12;
         u32 rxholdfull       : 1;
         u32 prxholdfull      : 1;
         u32 empty            : 1;
         u32 uarton           : 1;
#else
         u32 uarton           : 1;
         u32 empty            : 1;
         u32 prxholdfull      : 1;
         u32 rxholdfull       : 1;
         u32 reserved16       : 12;
         u32 endmarx          : 1;
         u32 endmatx          : 1;
         u32 testmode         : 1;
         u32 enbreakhalt      : 1;
         u32 endmatest        : 1;
         u32 endmaloop        : 1;
         u32 pulseopt2        : 1;
         u32 pulseopt1        : 1;
         u32 dtinvert         : 1;
         u32 distxd           : 1;
         u32 twostop          : 1;
         u32 loopback         : 1;
         u32 bit_7            : 1;
         u32 evenparity       : 1;
         u32 enparity         : 1;
         u32 enuart           : 1;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x40000040;
      }
};


// ------------------------------------------------------------------------
//
// UART Control 2 Register (section 15.5.2)
// Write-Only
//
//

union Uartcr2 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 baudrate         : 10;
         u32 reserved10       : 22;
#else
         u32 reserved10       : 22;
         u32 baudrate         : 10;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// UART DMA Control 1 Register (section 15.5.3)
// Write-Only
//
//

union Uartdmacr1 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 reserved0        : 2;
         u32 dmastartval      : 30;
#else
         u32 dmastartval      : 30;
         u32 reserved0        : 2;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// UART DMA Control 2 Register (section 15.5.4)
// Write-Only
//
//

union Uartdmacr2 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 dmalength        : 16;
         u32 reserved16       : 16;
#else
         u32 reserved16       : 16;
         u32 dmalength        : 16;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// UART DMA Count Register (section 15.5.5)
// Read-Only
//
//

union Uartdmacnt {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 dmacnt           : 16;
         u32 reserved16       : 16;
#else
         u32 reserved16       : 16;
         u32 dmacnt           : 16;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// UART Transmit Holding Register (section 15.5.6)
// Write-Only
//
//

union Uarttxhld {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 txdata           : 8;
         u32 brk              : 1;
         u32 reserved9        : 23;
#else
         u32 reserved9        : 23;
         u32 brk              : 1;
         u32 txdata           : 8;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// UART Receiver Holding Register (section 15.5.7)
// Read-Only
//
//

union Uartrxhld {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 rxdata           : 8;
         u32 reserved8        : 24;
#else
         u32 reserved8        : 24;
         u32 rxdata           : 8;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Combined UART registers
//
//

struct UartRegisters {
   public:
      Uartcr1 cr1;
      Uartcr2 cr2;
      Uartdmacr1 dmacr1;
      Uartdmacr2 dmacr2;
      Uartdmacnt dmacnt;
      Uarttxhld txhld;
      Uartrxhld rxhld;

   public:
      void Reset (void) {
         cr1.Reset ();
         cr2.Reset ();
         dmacr1.Reset ();
         dmacr2.Reset ();
         dmacnt.Reset ();
         txhld.Reset ();
         rxhld.Reset ();
      }
};


// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
//
// Interrupt registers
//
//

// ------------------------------------------------------------------------
//
// Interrupt Status/Clear/Enable 1 Register (section 8.3.1)
// Read-Only
//
//

union Int1 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 reserved0        : 5;
         u32 sibirqnegint     : 1;
         u32 sibirqposint     : 1;
         u32 sibsf1int        : 1;
         u32 sibsf0int        : 1;
         u32 telinint         : 1;
         u32 sndinint         : 1;
         u32 valtelnegint     : 1;
         u32 valtelposint     : 1;
         u32 valsndnegint     : 1;
         u32 valsndposint     : 1;
         u32 rsndclipint      : 1;
         u32 lsndclipint      : 1;
         u32 teldmacntint     : 1;
         u32 snddmacntint     : 1;
         u32 tel1_0int        : 1;
         u32 tel0_5int        : 1;
         u32 snd1_0int        : 1;
         u32 snd0_5int        : 1;
         u32 chierrint        : 1;
         u32 chiactint        : 1;
         u32 chiinintb        : 1;
         u32 chiininta        : 1;
         u32 chidmacntint     : 1;
         u32 chi1_0int        : 1;
         u32 chi0_5int        : 1;
         u32 dfint            : 1;
         u32 lcdint           : 1;
#else
         u32 lcdint           : 1;
         u32 dfint            : 1;
         u32 chi0_5int        : 1;
         u32 chi1_0int        : 1;
         u32 chidmacntint     : 1;
         u32 chiininta        : 1;
         u32 chiinintb        : 1;
         u32 chiactint        : 1;
         u32 chierrint        : 1;
         u32 snd0_5int        : 1;
         u32 snd1_0int        : 1;
         u32 tel0_5int        : 1;
         u32 tel1_0int        : 1;
         u32 snddmacntint     : 1;
         u32 teldmacntint     : 1;
         u32 lsndclipint      : 1;
         u32 rsndclipint      : 1;
         u32 valsndposint     : 1;
         u32 valsndnegint     : 1;
         u32 valtelposint     : 1;
         u32 valtelnegint     : 1;
         u32 sndinint         : 1;
         u32 telinint         : 1;
         u32 sibsf0int        : 1;
         u32 sibsf1int        : 1;
         u32 sibirqposint     : 1;
         u32 sibirqnegint     : 1;
         u32 reserved0        : 5;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};

// Masks for this register

#define INT1_SND0_5INT     0x00400000
#define INT1_SND1_0INT     0x00200000
#define INT1_SIBSF0        0x00000100
#define INT1_SIBSF1        0x00000080
#define INT1_SIBIRQPOSINT  0x00000040


// ------------------------------------------------------------------------
//
// Interrupt Status/Clear/Enable 2 Register (section 8.3.2)
// Read-Only
//
//

union Int2 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL

         u32 reserved0         : 12;
         u32 uartbdmahalfint   : 1;
         u32 uartbdmafullint   : 1;
         u32 uartbemptyint     : 1;
         u32 uartbtxoverrunint : 1;
         u32 uartbtxint        : 1;
         u32 uartbparityerrint : 1;
         u32 uartbbreakint     : 1;
         u32 uartbframeerrint  : 1;
         u32 uartbrxoverrunint : 1;
         u32 uartbrxint        : 1;
         u32 uartadmahalfint   : 1;
         u32 uartadmafullint   : 1;
         u32 uartaemptyint     : 1;
         u32 uartatxoverrunint : 1;
         u32 uartatxint        : 1;
         u32 uartaparityerrint : 1;
         u32 uartabreakint     : 1;
         u32 uartaframeerrint  : 1;
         u32 uartarxoverrunint : 1;
         u32 uartarxint        : 1;
#else

         u32 uartarxint        : 1;
         u32 uartarxoverrunint : 1;
         u32 uartaframeerrint  : 1;
         u32 uartabreakint     : 1;
         u32 uartaparityerrint : 1;
         u32 uartatxint        : 1;
         u32 uartatxoverrunint : 1;
         u32 uartaemptyint     : 1;
         u32 uartadmafullint   : 1;
         u32 uartadmahalfint   : 1;
         u32 uartbrxint        : 1;
         u32 uartbrxoverrunint : 1;
         u32 uartbframeerrint  : 1;
         u32 uartbbreakint     : 1;
         u32 uartbparityerrint : 1;
         u32 uartbtxint        : 1;
         u32 uartbtxoverrunint : 1;
         u32 uartbemptyint     : 1;
         u32 uartbdmafullint   : 1;
         u32 uartbdmahalfint   : 1;
         u32 reserved0         : 12;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};
#define INT2_UARTBTXINT         0x00010000
#define INT2_UARTBRXINT         0x00200000
#define INT2_UARTADMAHALFINT    0x00400000
#define INT2_UARTADMAFULLINT    0x00800000
#define INT2_UARTATXINT         0x04000000
#define INT2_UARTARXINT         0x80000000

// ------------------------------------------------------------------------
//
// Interrupt Status/Clear/Enable 3 Register (section 8.3.3)
// Read-Only
//
//

union Int3 {
   public:
      u32   value;
      struct {
         u32 mfioposint        : 32;
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Interrupt Status/Clear/Enable 4 Register (section 8.3.4)
// Read-Only
//
//

union Int4 {
   public:
      u32   value;
      struct {
         u32 mfionegint        : 32;
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Interrupt Status/Clear/Enable 5 Register (section 8.3.5)
// Read-Only
//
//

union Int5 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 ionegint          : 7;
         u32 ioposint          : 7;
         u32 negcarint         : 1;
         u32 poscarint         : 1;
         u32 carstint          : 1;
         u32 irconsmint        : 1;
         u32 spiemptyint       : 1;
         u32 sprrcvint         : 1;
         u32 spierrint         : 1;
         u32 spibufavailint    : 1;
         u32 negonbutnint      : 1;
         u32 posonbutnint      : 1;
         u32 negpwrokint       : 1;
         u32 pospwrokint       : 1;
         u32 negpwrint         : 1;
         u32 pospwrint         : 1;
         u32 stptimerint       : 1;
         u32 perint            : 1;
         u32 alarmint          : 1;
         u32 rtcint            : 1;
#else
         u32 rtcint            : 1;
         u32 alarmint          : 1;
         u32 perint            : 1;
         u32 stptimerint       : 1;
         u32 pospwrint         : 1;
         u32 negpwrint         : 1;
         u32 pospwrokint       : 1;
         u32 negpwrokint       : 1;
         u32 posonbutnint      : 1;
         u32 negonbutnint      : 1;
         u32 spibufavailint    : 1;
         u32 spierrint         : 1;
         u32 sprrcvint         : 1;
         u32 spiemptyint       : 1;
         u32 irconsmint        : 1;
         u32 carstint          : 1;
         u32 poscarint         : 1;
         u32 negcarint         : 1;
         u32 ioposint          : 7;
         u32 ionegint          : 7;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};

// Masks for this register

#define INT5_IONEG0        0x00000001
#define INT5_IONEG5        0x00000020
#define INT5_IONEG6        0x00000040
#define INT5_IOPOS0        0x00000080
#define INT5_IOPOS5        0x00001000
#define INT5_IOPOS6        0x00002000
#define INT5_NEGONBUTNINT  0x00400000
#define INT5_POSONBUTNINT  0x00800000
#define INT5_PERINT        0x20000000
#define INT5_ALARMINT      0x40000000
#define INT5_RTCINT        0x80000000


// ------------------------------------------------------------------------
//
// Interrupt Status 6 Register (section 8.3.6)
// Read-Only
//
//

union Intstat6 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 reserved0         : 2;
         u32 intvect           : 4;
         u32 reserved6         : 24;
         u32 irqlow            : 1;
         u32 irqhigh           : 1;
#else
         u32 irqhigh           : 1;
         u32 irqlow            : 1;
         u32 reserved6         : 24;
         u32 intvect           : 4;
         u32 reserved0         : 2;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Interrupt Enable 6 Register (section 8.3.17)
// Read-Write
//
//

union Inten6 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 prioritymask      : 16;
         u32 irqtest           : 1;
         u32 irqpritest        : 1;
         u32 globalen          : 1;
         u32 reserved19        : 13;
#else
         u32 reserved19        : 13;
         u32 globalen          : 1;
         u32 irqpritest        : 1;
         u32 irqtest           : 1;
         u32 prioritymask      : 16;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Combined Interrupt registers
//
//

struct IntRegisters {
   public:
      Int1 intstat1;
      Int2 intstat2;
      Int3 intstat3;
      Int4 intstat4;
      Int5 intstat5;
      Intstat6 intstat6;
      Int1 inten1;
      Int2 inten2;
      Int3 inten3;
      Int4 inten4;
      Int5 inten5;
      Inten6 inten6;


   public:
      void Reset (void) {
         intstat1.Reset ();
         intstat2.Reset ();
         intstat3.Reset ();
         intstat4.Reset ();
         intstat5.Reset ();
         intstat6.Reset ();
         inten1.Reset ();
         inten2.Reset ();
         inten3.Reset ();
         inten4.Reset ();
         inten5.Reset ();
         inten6.Reset ();
      }
};


// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
//
// Clock registers
//
//

// ------------------------------------------------------------------------
//
// Clock Control Register (section 6.3.1)
// Read-Write
//
//

union Clkcr {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 enuartbclk         : 1;
         u32 enuartaclk         : 1;
         u32 enirclk            : 1;
         u32 encserclk          : 1;
         u32 cserdiv            : 3;
         u32 csersel            : 1;
         u32 sibmclkdiv         : 3;
         u32 ensibmclk          : 1;
         u32 reserved12         : 1;
         u32 sibmclkdir         : 1;
         u32 enfasttimerclk     : 1;
         u32 entimerclk         : 1;
         u32 enspiclk           : 1;
         u32 enmbusclk          : 1;
         u32 envidclk           : 1;
         u32 enchimclk          : 1;
         u32 chiclkdir          : 1;
         u32 chimclksel         : 1;
         u32 clktestselsib      : 1;
         u32 enclktest          : 1;
         u32 chiclkdiv          : 8;
#else
         u32 chiclkdiv          : 8;
         u32 enclktest          : 1;
         u32 clktestselsib      : 1;
         u32 chimclksel         : 1;
         u32 chiclkdir          : 1;
         u32 enchimclk          : 1;
         u32 envidclk           : 1;
         u32 enmbusclk          : 1;
         u32 enspiclk           : 1;
         u32 entimerclk         : 1;
         u32 enfasttimerclk     : 1;
         u32 sibmclkdir         : 1;
         u32 reserved12         : 1;
         u32 ensibmclk          : 1;
         u32 sibmclkdiv         : 3;
         u32 csersel            : 1;
         u32 cserdiv            : 3;
         u32 encserclk          : 1;
         u32 enirclk            : 1;
         u32 enuartaclk         : 1;
         u32 enuartbclk         : 1;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000020;
      }
};


// ------------------------------------------------------------------------
//
// Combined Clock registers
//
//

struct ClkRegisters {
   public:
      Clkcr clkcr;

   public:
      void Reset (void) {
         clkcr.Reset ();
      }
};


// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
//
// IO pin registers
//
//

// ------------------------------------------------------------------------
//
// IO Control Register (section 9.3.1)
// Read-Write
//
//

union Iocr {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 iodin              : 7;
         u32 reserved7          : 1;
         u32 iodout             : 7;
         u32 reserved15         : 1;
         u32 iodirec            : 7;
         u32 reserved23         : 1;
         u32 iodebsel           : 7;
         u32 reserved31         : 1;
#else
         u32 reserved31         : 1;
         u32 iodebsel           : 7;
         u32 reserved23         : 1;
         u32 iodirec            : 7;
         u32 reserved15         : 1;
         u32 iodout             : 7;
         u32 reserved7          : 1;
         u32 iodin              : 7;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// MFIO Data Output Register (section 9.3.2)
// Read-Write
//
//

union Mfiodout {
   public:
      u32   value;
      struct {
         u32 mfiodout           : 32;
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// MFIO Data Direction Register (section 9.3.3)
// Read-Write
//
//

union Mfiodir {
   public:
      u32   value;
      struct {
         u32 mfiodirec         : 32;
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// MFIO Data Input Register (section 9.3.4)
// Read-Write
//
//

union Mfiodin {
   public:
      u32   value;
      struct {
         u32 mfiodin            : 32;
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// MFIO Select Register (section 9.3.5)
// Read-Write
//
//

union Mfiosel {
   public:
      u32   value;
      struct {
         u32 mfiosel            : 32;
      } bits;

   public:
      void Reset (void) {
         value = 0xf20f0fff;
      }
};


// ------------------------------------------------------------------------
//
// IO Power-down Register (section 9.3.6)
// Read-Write
//
//

union Iopd {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 iopd               : 7;
         u32 reserved7          : 25;
#else
         u32 reserved7          : 25;
         u32 iopd               : 7;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x0000007F;
      }
};


// ------------------------------------------------------------------------
//
// MFIO Power-down Register (section 9.3.6)
// Read-Write
//
//

union Mfiopd {
   public:
      u32   value;
      struct {
         u32 mfiopd             : 32;
      } bits;

   public:
      void Reset (void) {
         value = 0xfaf03ffc;
      }
};


// ------------------------------------------------------------------------
//
// Combined IO registers
//
//

struct IORegisters {
   public:
      Iocr iocr;
      Mfiodout dout;
      Mfiodir dir;
      Mfiodin din;
      Mfiosel sel;
      Iopd iopd;
      Mfiopd mfiopd;

   public:
      void Reset (void) {
         iocr.Reset ();
         dout.Reset ();
         dir.Reset ();
         din.Reset ();
         sel.Reset ();
         iopd.Reset ();
         mfiopd.Reset ();
      }
};


// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
//
// IR registers
//
//

// ------------------------------------------------------------------------
//
// IR Control 1 Register (section 10.5.1)
// Read-Write
//
//

union Ircr1 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 enconsm            : 1;
         u32 enstate            : 1;
         u32 rxpwr              : 1;
         u32 dtinvert           : 1;
         u32 testir             : 1;
         u32 reserved5          : 11;
         u32 baudval            : 8;
         u32 cardet             : 1;
         u32 reserved25         : 7;
#else
         u32 reserved25         : 7;
         u32 cardet             : 1;
         u32 baudval            : 8;
         u32 reserved5          : 11;
         u32 testir             : 1;
         u32 dtinvert           : 1;
         u32 rxpwr              : 1;
         u32 enstate            : 1;
         u32 enconsm            : 1;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// IR Control 2 Register (section 10.5.2)
// Write-only
//
//

union Ircr2 {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 waitval            : 8;
         u32 delayval           : 8;
         u32 ontime             : 8;
         u32 per                : 8;
#else
         u32 per                : 8;
         u32 ontime             : 8;
         u32 delayval           : 8;
         u32 waitval            : 8;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// IR Holding Register (section 10.5.3)
// Write-only
//
//

union Irhld {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 txhold             : 16;
         u32 reserved16         : 16;
#else
         u32 reserved16         : 16;
         u32 txhold             : 16;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Combined IR registers
//
//

struct IRRegisters {
   public:
      Ircr1 cr1;
      Ircr2 cr2;
      Irhld hld;

   public:
      void Reset (void) {
         cr1.Reset ();
         cr2.Reset ();
         hld.Reset ();
      }
};


// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
//
// Timer registers
//
//

// ------------------------------------------------------------------------
//
// RTC High Register (section 14.4.1)
// Read-only
//
//

union Rtchi {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 rtc                : 8;
         u32 reserved8          : 24;
#else
         u32 reserved8          : 24;
         u32 rtc                : 8;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// RTC Low Register (section 14.4.1)
// Read-only
//
//

union Rtclo {
   public:
      u32   value;
      struct {
         u32 rtc                : 32;
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Alarm High Register (section 14.4.2)
// Read-only
//
//

union Almhi {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 alarm              : 8;
         u32 reserved8          : 24;
#else
         u32 reserved8          : 24;
         u32 alarm              : 8;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Alarm Low Register (section 14.4.2)
// Read-only
//
//

union Almlo {
   public:
      u32   value;
      struct {
         u32 alarm              : 32;
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Timer Control Register (section 14.4.3)
// Read-write
//
//

union Tmrcr {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 enrtctst           : 1;
         u32 entestclk          : 1;
         u32 testc8ms           : 1;
         u32 rtcclr             : 1;
         u32 enpertimer         : 1;
         u32 freezetimer        : 1;
         u32 freezertc          : 1;
         u32 freezepre          : 1;
         u32 reserved8          : 24;
#else
         u32 reserved8          : 24;
         u32 freezepre          : 1;
         u32 freezertc          : 1;
         u32 freezetimer        : 1;
         u32 enpertimer         : 1;
         u32 rtcclr             : 1;
         u32 testc8ms           : 1;
         u32 entestclk          : 1;
         u32 enrtctst           : 1;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Periodic Timer Register (section 14.4.4)
// Read-write
//
//

union Pertmr {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 perval             : 16;
         u32 percnt             : 16;
#else
         u32 percnt             : 16;
         u32 perval             : 16;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Combined Timer registers
//
//

struct TmrRegisters {
   public:
      Rtchi rtchi;
      Rtclo rtclo;
      Almhi almhi;
      Almlo almlo;
      Tmrcr cr;
      Pertmr pertmr;

   public:
      void Reset (void) {
         rtchi.Reset ();
         rtclo.Reset ();
         almhi.Reset ();
         almlo.Reset ();
         cr.Reset ();
         pertmr.Reset ();
      }
};


// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
//
// SPI registers
//
//

// ------------------------------------------------------------------------
//
// SPI Control Register (section 13.3.1)
// Read-write
//
//

union Spicr {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 enspi              : 1;
         u32 lsb                : 1;
         u32 word               : 1;
         u32 reserved3          : 1;
         u32 clkpol             : 1;
         u32 phapol             : 1;
         u32 reserved6          : 2;
         u32 baudrate           : 4;
         u32 delayval           : 4;
         u32 empty              : 1;
         u32 spion              : 1;
         u32 reserved18         : 14;
#else
         u32 reserved18         : 14;
         u32 spion              : 1;
         u32 empty              : 1;
         u32 delayval           : 4;
         u32 baudrate           : 4;
         u32 reserved6          : 2;
         u32 phapol             : 1;
         u32 clkpol             : 1;
         u32 reserved3          : 1;
         u32 word               : 1;
         u32 lsb                : 1;
         u32 enspi              : 1;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// SPI Transmitter/Receiver Holding (section 13.3.2)
// Read-write
//
//

union Spidata {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 txdata             : 16;
         u32 reserved16         : 16;
#else
         u32 reserved16         : 16;
         u32 txdata             : 16;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Combined SPI registers
//
//

struct SpiRegisters {
   public:
      Spicr cr;
      Spidata data;

   public:
      void Reset (void) {
         cr.Reset ();
         data.Reset ();
      }
};


// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
//
// SIU test registers
//
//

// ------------------------------------------------------------------------
//
// SIU Test Register (section 5.3.1)
// Read-write
//
//

union Siutest {
   public:
      u32   value;
      struct {
#ifdef   HOST_BITFIELDS_EL
         u32 dmatestwr          : 6;
         u32 ennotimetest       : 1;
         u32 endmatest          : 1;
         u32 reserved8          : 24;
#else
         u32 reserved8          : 24;
         u32 endmatest          : 1;
         u32 ennotimetest       : 1;
         u32 dmatestwr          : 6;
#endif
      } bits;

   public:
      void Reset (void) {
         value = 0x00000000;
      }
};


// ------------------------------------------------------------------------
//
// Combined SIU registers
//
//

struct SiuRegisters {
   public:
      Siutest test;

   public:
      void Reset (void) {
         test.Reset ();
      }
};



// ------------------------------------------------------------------------
//
// Special internal state vars.  These are vars that have to be persistent
// and represent summary state info from the regs.  This isn't the cleanest
// place to add these but...
//
//

struct Pr31700InternalRegisters {
   public:
      bool halted;         // Persistent 'halted' state.  Set if either
                           // in a power down (vccon, pwrcs = 0) or in
                           // a powered up doze (stopcpu = 1).  Provides
                           // a cheaper summary for checking

      bool clear_onbutn;   // Set only at hard reset.  If this is set on a
                           // write to the powcr, we clear onbutn, otherwise
                           // we preserve it
   public:
      void Reset (void) {
         halted = false;
         clear_onbutn = true;
      }
};


// ------------------------------------------------------------------------
// ------------------------------------------------------------------------
//
// Combine all registers
//
//

struct Pr31700Registers {
   public:
      BiuRegisters biu;
      VidRegisters vid;
      SibRegisters sib;
      PowRegisters pow;
      UartRegisters uartA;
      UartRegisters uartB;
      IntRegisters irq;
      ClkRegisters clk;
      IORegisters io;
      IRRegisters ir;
      TmrRegisters tmr;
      SpiRegisters spi;
      SiuRegisters siu;

      Pr31700InternalRegisters internal;

   public:
      void Reset (void) {
         biu.Reset ();
         vid.Reset ();
         sib.Reset ();
         pow.Reset ();
         uartA.Reset ();
         uartB.Reset ();
         irq.Reset ();
         clk.Reset ();
         io.Reset ();
         ir.Reset ();
         tmr.Reset ();
         spi.Reset ();
         siu.Reset ();

         internal.Reset ();
      }
};

#endif
